@@ -4,11 +4,10 @@ import itertools
 import logging
 import queue
 import threading
-from asyncio.events import AbstractEventLoop
 from typing import Callable, Generic, Optional, TypeVar, Union
 
 from aiortc import RTCPeerConnection, RTCSessionDescription
-from aiortc.contrib.media import MediaPlayer, MediaRecorder
+from aiortc.contrib.media import MediaPlayer, MediaRecorder, MediaRelay
 
 from .process import (
     AsyncAudioProcessTrack,
@@ -20,6 +19,7 @@ from .process import (
     VideoTransformerBase,
 )
 from .receive import AudioReceiver, VideoReceiver
+from .relay import get_relay
 
 __all__ = [
     "AudioProcessorBase",
@@ -62,6 +62,7 @@ async def _process_offer(
     mode: WebRtcMode,
     pc: RTCPeerConnection,
     offer: RTCSessionDescription,
+    relay: MediaRelay,
     player_factory: Optional[MediaPlayerFactory],
     in_recorder_factory: Optional[MediaRecorderFactory],
     out_recorder_factory: Optional[MediaRecorderFactory],
@@ -116,7 +117,8 @@ async def _process_offer(
                             AudioTrack,
                         )
                         output_track = AudioTrack(
-                            track=input_track, processor=audio_processor
+                            track=relay.subscribe(input_track),
+                            processor=audio_processor,
                         )
                         logger.info("Add the audio track with processor to %s", pc)
                     else:
@@ -138,7 +140,8 @@ async def _process_offer(
                             VideoTrack,
                         )
                         output_track = VideoTrack(
-                            track=input_track, processor=video_processor
+                            track=relay.subscribe(input_track),
+                            processor=video_processor,
                         )
                         logger.info("Add the video track with processor to %s", pc)
                     else:
@@ -150,13 +153,13 @@ async def _process_offer(
                         "Either factory must be set."
                     )
 
-                pc.addTrack(output_track)
+                pc.addTrack(relay.subscribe(output_track))
                 if out_recorder:
                     logger.info("Track %s is added to out_recorder", output_track.kind)
-                    out_recorder.addTrack(output_track)
+                    out_recorder.addTrack(relay.subscribe(output_track))
                 if in_recorder:
                     logger.info("Track %s is added to in_recorder", input_track.kind)
-                    in_recorder.addTrack(input_track)
+                    in_recorder.addTrack(relay.subscribe(input_track))
 
                 @input_track.on("ended")
                 async def on_ended():
@@ -296,7 +299,7 @@ webrtc_thread_id_generator = itertools.count()
 
 class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
     _webrtc_thread: Union[threading.Thread, None]
-    _loop: Union[AbstractEventLoop, None]
+    _loop: asyncio.AbstractEventLoop
     _answer_queue: queue.Queue
     _video_processor: Optional[VideoProcessorT]
     _audio_processor: Optional[AudioProcessorT]
@@ -321,6 +324,7 @@ class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
 
     def __init__(
         self,
+        loop: asyncio.AbstractEventLoop,
         mode: WebRtcMode,
         player_factory: Optional[MediaPlayerFactory] = None,
         in_recorder_factory: Optional[MediaRecorderFactory] = None,
@@ -336,7 +340,7 @@ class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
         audio_receiver_size: int = 4,
     ) -> None:
         self._webrtc_thread = None
-        self._loop = None
+        self._loop = loop
         self.pc = RTCPeerConnection()
         self._answer_queue = queue.Queue()
 
@@ -367,15 +371,6 @@ class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
             )
         except Exception as e:
             logger.warn("An error occurred in the WebRTC worker thread: %s", e)
-
-            if self._loop:
-                logger.warn("An event loop exists. Clean up it.")
-                loop = self._loop
-                loop.run_until_complete(self.pc.close())
-                loop.run_until_complete(loop.shutdown_asyncgens())
-                loop.close()
-                logger.warn("Event loop %s cleaned up.", loop)
-
             self._answer_queue.put(e)  # Send the error object to the main thread
 
     def _webrtc_thread_impl(
@@ -387,8 +382,7 @@ class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
             "_webrtc_thread_impl starts",
         )
 
-        loop = asyncio.new_event_loop()
-        self._loop = loop
+        loop = self._loop
 
         offer = RTCSessionDescription(sdp, type_)
 
@@ -414,6 +408,8 @@ class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
         self._video_receiver = video_receiver
         self._audio_receiver = audio_receiver
 
+        relay = get_relay(loop=loop)
+
         @self.pc.on("iceconnectionstatechange")
         async def on_iceconnectionstatechange():
             iceConnectionState = self.pc.iceConnectionState
@@ -425,6 +421,7 @@ class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
                 self.mode,
                 self.pc,
                 offer,
+                relay=relay,
                 player_factory=self.player_factory,
                 in_recorder_factory=self.in_recorder_factory,
                 out_recorder_factory=self.out_recorder_factory,
@@ -437,15 +434,6 @@ class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
             )
         )
 
-        try:
-            loop.run_forever()
-        finally:
-            logger.debug("Event loop %s has stopped.", loop)
-            loop.run_until_complete(self.pc.close())
-            loop.run_until_complete(loop.shutdown_asyncgens())
-            loop.close()
-            logger.debug("Event loop %s cleaned up.", loop)
-
     def process_offer(
         self, sdp, type_, timeout: Union[float, None] = 10.0
     ) -> RTCSessionDescription:
@@ -486,8 +474,6 @@ class WebRtcWorker(Generic[VideoProcessorT, AudioProcessorT]):
 
     def stop(self, timeout: Union[float, None] = 1.0):
         self._unset_processors()
-        if self._loop:
-            self._loop.stop()
         if self._webrtc_thread:
             self._webrtc_thread.join(timeout=timeout)
 
